package com.hzya.frame.grpU8.nxproof.gzlwfml.service.impl; 
import cn.hutool.core.lang.Assert;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DS;
import com.hzya.frame.grpU8.nxproof.gzlwfml.entity.GzLwfNrEntity;
import com.hzya.frame.grpU8.nxproof.gzlwfml.service.IGzLwfNrService;
import com.hzya.frame.uuid.UUIDUtils;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import com.hzya.frame.web.exception.BaseSystemException;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.hzya.frame.grpU8.nxproof.gzlwfml.entity.GzLwfMlEntity;
import com.hzya.frame.grpU8.nxproof.gzlwfml.service.IGzLwfMlService;
import com.hzya.frame.grpU8.nxproof.gzlwfml.dao.IGzLwfMlDao;
import com.hzya.frame.basedao.service.impl.BaseService;

import java.util.List;

/**
 * @description: 劳务费目录表  service
 * @tableName:   GZ_LWFML 
 * @entityName:  GzLwfMlEntity
 * @author: 	 gjh
 * @history:	 1.0
 */
@Service(value="GZ_LWFMLService")
public class  GzLwfMlServiceImpl extends BaseService< GzLwfMlEntity,String> implements IGzLwfMlService{
	
	protected IGzLwfMlDao  gzlwfmlDao;
	@Autowired
	private IGzLwfNrService gzLwfNrService;
	@Autowired
	public void setGzLwfMlDao(IGzLwfMlDao dao) {
		this.gzlwfmlDao = dao;
		this.dao=dao;
	}

	/**
	 * 查询最大id
	 *
	 * @param entity
	 * @return
	 */
	@DS("#entity.dataSourceCode")
	@Override
	public String queryMaxMxdh(GzLwfMlEntity entity) {
		Assert.notNull(entity,"参数不能为空");
		Assert.notEmpty(entity.getGSDM(),"gsdm参数不能为空");
		Assert.notEmpty(entity.getFFND(),"ffdn参数不能为空");
		List<GzLwfMlEntity> lwfList = gzlwfmlDao.queryList(entity, "entity_list_maxMlid");
		if (CollectionUtils.isNotEmpty(lwfList)){
			logger.info("查询到的最大mxdh:{}",lwfList.get(0).getMXDH());
			return lwfList.get(0).getMXDH();
		}
		return null;
	}

	/**
	 * 劳务费保存
	 *
	 * @param jsonObject
	 * @return
	 */
	@Override
	public JsonResultEntity saveLwf(JSONObject jsonObject) {
		GzLwfMlEntity lwfMlEntity = getData("jsonStr", jsonObject, GzLwfMlEntity.class);
		try {
			checkParams(lwfMlEntity,"save");
		}catch (Exception e){
			BaseResult.getFailureMessageEntity(e.getMessage());
		}
		//处理最大mxdh
		lwfMlEntity.setMXDH(queryMaxMxdh(lwfMlEntity));
		GzLwfMlEntity lwf = gzlwfmlDao.save(lwfMlEntity);
		//保存子表
		for (GzLwfNrEntity gzLwfNrEntity : lwfMlEntity.getLwfnrList()) {
			gzLwfNrEntity.setMXDH(lwfMlEntity.getMXDH());
			gzLwfNrEntity.setBillGUID(UUIDUtils.getUUID());
			gzLwfNrEntity.setDataSourceCode(lwfMlEntity.getDataSourceCode());
			gzLwfNrService.saveLwfNr(gzLwfNrEntity);
		}
		return BaseResult.getSuccessMessageEntity("保存单据成功");
	}

	/**
	 * 劳务费处理
	 *
	 * @param jsonObject
	 * @return
	 */
	@Override
	public JsonResultEntity updateLwf(JSONObject jsonObject) {
		GzLwfMlEntity lwfMlEntity = getData("jsonStr", jsonObject, GzLwfMlEntity.class);
		try {
			checkParams(lwfMlEntity,"update");
		}catch (Exception e){
			BaseResult.getFailureMessageEntity(e.getMessage());
		}
		gzlwfmlDao.update(lwfMlEntity);
		//处理子表数据 , 先删掉，重新保存
		GzLwfNrEntity gzLwfNrEntity = new GzLwfNrEntity();
		gzLwfNrEntity.setDataSourceCode(lwfMlEntity.getDataSourceCode());
		gzLwfNrEntity.setMXDH(lwfMlEntity.getMXDH());
		gzLwfNrEntity.setGSDM(lwfMlEntity.getGSDM());
		gzLwfNrEntity.setFFND(lwfMlEntity.getFFND());
		int rows = gzLwfNrService.deleteLwfNr(gzLwfNrEntity);
		logger.info("根据mxdh{},gsdm:{},ffnd:{},删除子表，受影响行数为:{}",lwfMlEntity.getMXDH(),lwfMlEntity.getGSDM(),lwfMlEntity.getFFND(),rows);
		return BaseResult.getSuccessMessageEntity("修改单据成功");
	}

	private void checkParams(GzLwfMlEntity entity, String type){
		Assert.notNull(entity,"参数不能为空");
		Assert.notEmpty(entity.getDataSourceCode(),"数据源编码不能为空");
		Assert.notEmpty(entity.getLclb(),"流程类别不能为空");
		Assert.notEmpty(entity.getDJBH(),"单据编号不能为空");
		if ("save".equals(type)){
			Assert.notEmpty(entity.getGSDM(),"公司代码不能为空");
			Assert.notEmpty(entity.getFFND(),"发放年度不能为空");
			Assert.notEmpty(entity.getFFND(),"发放年度不能为空");
			if (CollectionUtils.isEmpty(entity.getLwfnrList())){
				throw new BaseSystemException("劳务费明细列表不能为空");
			}
			if (CollectionUtils.isEmpty(entity.getLwfsdsList())){
				throw new BaseSystemException("劳务费税率列表不能为空");
			}
		}else if ("update".equals(type)){
			Assert.notEmpty(entity.getMXDH(),"mxdh不能为空");
		}
	}
}
